

%% use simulated data for now
clear; clc;
% addpath('C:\Users\Ole\Dropbox\Research\ABCForecasting\MISC_fn');
addpath('E:\Dropbox\Research\ABCForecasting\MISC_fn');
% load('simdata.mat');

%% loading in real data!
load('rtrvbv.mat');

%take a subset - 1750 obs with a 250 obs hold out for prediction
TT=1750;
m=250;
T=TT-m;
yout=RT(end-m+1:end);
BVout=BV(end-m+1:end);
RVout=RV(end-m+1:end);
y=RT(end-TT+1:end-m);
RV=RV(end-TT+1:end-m);
BV=BV(end-TT+1:end-m);
lnBV=log(BV);
lnRV=log(RV);
JV=RV-BV;
JV(JV<=0)=0; %take max(RV-BV,0);
sRT=y./sqrt(BV);
%% estimate the realized GARCH model - log-linear spec
tic;
loglike=@(p) -LogRGARCH_ll(y, lnRV, p);
p0=[0,0.8,0.1,0,1,0,0,0.2]';
A=[0,0,1,1,0,0,0,0];
b=0.99;
Aeq=[]; beq=[];
lb=[-Inf,0,0,-Inf,-Inf,-Inf,-Inf,0];
ub=[+Inf,1,1,+Inf,+Inf,+Inf,+Inf,+Inf];
phat = fmincon(loglike,p0,A,b,Aeq,beq,lb,ub); %MLE estimate using the data

dat_score=LogRGARCH_scores(y, lnRV ,phat);
sJV=sign(y).*sqrt(JV);
dat_stat=[dat_score,kurtosis(lnBV),mean(sJV),var(JV),corr(JV(1:end-1),JV(2:end))];

%% setting up ABC stuffs
%number of ABC draws per run - a function of T
eps=50/(T.^(3/2));
P=500;
% reps=50000; %do this for now.
reps=round(P/eps);
%storage vector for the summary stats
sim_stat=zeros(reps,length(dat_stat));

%% simulating from the prior
mup=-1+(1-(-1))*rand(reps,1);
sigp=0.5+(3-0.5)*rand(reps,1);
del0=0.001+(0.3-0.001)*rand(reps,1);
ap=0.001+(0.2-0.001)*rand(reps,1);
bp=0.5+(1-ap-0.5).*rand(reps,1);
psi0=-0.5+(0.5-(-0.5))*rand(reps,1);
psi1=0.5+(1.5-0.5)*rand(reps,1);
sigbv=0.001+(1-0.001)*rand(reps,1);
omgv=-1+(1-(-1))*rand(reps,1);
rho=0.5+(0.99-0.5)*rand(reps,1);
sigv=0.001+(0.3-0.001)*rand(reps,1);
alpha=1.5+(2-1.5)*rand(reps,1);

parfor i=1:reps
   [ sim_r, sim_lnBV, sim_x, sim_Jp, sim_delt ] = gen_RTBVModel( T, [mup(i),sigp(i),del0(i),ap(i),bp(i),psi0(i),psi1(i),sigbv(i),omgv(i),rho(i),sigv(i),alpha(i)] ); 
   sim_lnRV=log(exp(sim_lnBV)+sim_Jp.^2);
   sim_JV=sim_Jp.^2;
   sim_score=LogRGARCH_scores(sim_r, sim_lnRV ,phat);
   sim_sJV=sign(sim_r).*sqrt(sim_JV);
   stat=[sim_score,kurtosis(sim_lnBV),mean(sim_sJV),var(sim_JV),corr(sim_JV(1:end-1),sim_JV(2:end))];
   sim_stat(i,:)=stat;
end

%trapping out NaN records
theta_draw=[mup,sigp,del0,ap,bp,psi0,psi1,sigbv,omgv,rho,sigv,alpha];
theta_draw(any(isnan(sim_stat), 2), :)=[];
sim_stat(any(isnan(sim_stat), 2), :)=[];

%trapping out Inf records
theta_draw(any(isinf(sim_stat), 2), :)=[];
sim_stat(any(isinf(sim_stat), 2), :)=[];

[sstat_dist]=sstatDist_Euclidean(sim_stat, dat_stat, length(sim_stat));
theta_dist=[theta_draw,sstat_dist];
theta_dist_sorted=sortrows(theta_dist,13);
theta_dist=theta_dist_sorted(1:P,1:12);

ABCtime=toc;
save('realdata_ABC_RealizedGARCH.mat');

%%
figure;
subplot(2,2,1); plot((-1:2/99:1),ksdensity(theta_dist(:,1),(-1:2/99:1))); title('mup');
subplot(2,2,2); plot((0.5:2.5/99:3),ksdensity(theta_dist(:,2),(0.5:2.5/99:3))); title('sigp');
subplot(2,2,3); plot((0:.3/99:.3),ksdensity(theta_dist(:,3),(0:.3/99:.3))); title('del0');
subplot(2,2,4); plot((0:.2/99:.2),ksdensity(theta_dist(:,4),(0:.2/99:.2))); title('ap');

figure;
subplot(2,2,1); plot((0.5:0.5/99:1),ksdensity(theta_dist(:,5),(0.5:0.5/99:1))); title('bp');
subplot(2,2,2); plot((-0.5:1/99:0.5),ksdensity(theta_dist(:,6),(-0.5:1/99:0.5))); title('psi0');
subplot(2,2,3); plot((0.5:1/99:1.5),ksdensity(theta_dist(:,7),(0.5:1/99:1.5))); title('psi1');
subplot(2,2,4); plot((0:1/99:1),ksdensity(theta_dist(:,8),(0:1/99:1))); title('sigbv');

figure;
subplot(2,2,1); plot((-1:2/99:1),ksdensity(theta_dist(:,9),(-1:2/99:1))); title('omgv');
subplot(2,2,2); plot((0.5:0.49/99:0.99),ksdensity(theta_dist(:,10),(0.5:0.49/99:0.99))); title('rho');
subplot(2,2,3); plot((0:.3/99:.3),ksdensity(theta_dist(:,11),(0:.3/99:.3))); title('sigv');
subplot(2,2,4); plot((1.5:0.5/99:2),ksdensity(theta_dist(:,12),(1.5:0.5/99:2))); title('alpha');

%% regression adjustment for each column of draw

[~,r]=size(theta_draw);
theta_reg_sorted=zeros(P,r);

for i=1:r
    [fp_dist]= sstatDist_reg(sim_stat, dat_stat, theta_draw(:,i));
    tmp=sortrows([theta_draw(:,i),fp_dist],2);
    theta_reg_sorted(:,i)=tmp(1:P,1);
end
save('realdata_ABC_RealizedGARCH.mat');
% %% plot the kernel densities
% 
% figure;
% subplot(2,2,1); plot((-1:2/99:1),ksdensity(theta_reg_sorted(:,1),(-1:2/99:1))); title('mup=0');
% subplot(2,2,2); plot((1:2/99:3),ksdensity(theta_reg_sorted(:,2),(1:2/99:3))); title('sigp=2');
% subplot(2,2,3); plot((0:.3/99:.3),ksdensity(theta_reg_sorted(:,3),(0:.3/99:.3))); title('del0=0.1');
% subplot(2,2,4); plot((0:.2/99:.2),ksdensity(theta_reg_sorted(:,4),(0:.2/99:.2))); title('ap=0.03');
% 
% figure;
% subplot(2,2,1); plot((0:1/99:1),ksdensity(theta_reg_sorted(:,5),(0:1/99:1))); title('bp=0.95');
% subplot(2,2,2); plot((-1:2/99:1),ksdensity(theta_reg_sorted(:,6),(-1:2/99:1))); title('psi0=0');
% subplot(2,2,3); plot((0:2/99:2),ksdensity(theta_reg_sorted(:,7),(0:2/99:2))); title('psi1=1');
% subplot(2,2,4); plot((0:1/99:1),ksdensity(theta_reg_sorted(:,8),(0:1/99:1))); title('sigbv=0.5');
% 
% figure;
% subplot(2,2,1); plot((-1:2/99:1),ksdensity(theta_reg_sorted(:,9),(-1:2/99:1))); title('omgv=0');
% subplot(2,2,2); plot((0.5:0.49/99:0.99),ksdensity(theta_reg_sorted(:,10),(0.5:0.49/99:0.99))); title('rho0.95');
% subplot(2,2,3); plot((0:.3/99:.3),ksdensity(theta_reg_sorted(:,11),(0:.3/99:.3))); title('sigv=0.1');
% subplot(2,2,4); plot((1.5:0.5/99:2),ksdensity(theta_reg_sorted(:,12),(1.5:0.5/99:2))); title('alpha=1.8');

