function [ pred_r, pred_bv ] = predY_BPF_SVaStabJump( ygrid, lnBVgrid, m, y, lnBV, theta )

%theta=[mup,sigp,del0,ap,bp,psi0,psi1,sigbv,omgv,rho,sigv,alpha]
mup=theta(1); sigp=theta(2); del0=theta(3);
ap=theta(4); bp=theta(5); psi0=theta(6);
psi1=theta(7); sigbv=theta(8); omgv=theta(9);
rho=theta(10); sigv=theta(11); alpha=theta(11);
deli=del0*(1-ap-bp);

seed=123456;
rng(seed);

pred_r=zeros(m,length(ygrid));
pred_bv=zeros(m,length(lnBVgrid));

%set up the relevant matrices
T=length(y);
Tin=T-m;
i=1;
N=5000;%number of particles
ind=linspace(1,N,N);

pd=makedist('Stable','alpha',alpha,'beta',1,'gam',1,'delta',0);
vt=random(pd,T,N);
ut=rand(T,N);
Zp=normrnd(mup,sigp,T,N);

x0=omgv/(1-rho);

%repeat the loop for t=1:T
for t=1:T
    %sample new particles for time t
    if(t==1)
        mux=omgv+rho*x0;
        delt=del0;
    else
        mux=omgv+rho*filtered_states(1,:);
        delt=deli+ap*filtered_states(2,:)+bp*delt_lag;
    end
    new_x=mux+sigv*vt(t,:);
    new_N=1*(ut(t,:)<=delt);
    new_J=new_N.*Zp(t,:);
    new_states=[new_x;new_N;new_J];
      
    %compute the particle weights - for resampling
    lpdf_r=-0.5*log(2*pi)-0.5.*new_x-0.5.*((y(t)-new_J).^2)./exp(new_x);
    lpdf_bv=-0.5*log(2*pi)-log(sigbv)-0.5.*(((lnBV(t)-psi0-psi1.*new_x)/sigbv).^2);
    incr_weight=lpdf_r+lpdf_bv;
    lw=incr_weight-log(N);
    [norm_weight,~]=normLogWeights(lw);
    if(sum(norm_weight==0)==N || sum(isnan(norm_weight)>0))
       if(t==1)
            norm_weight=(1/N)*ones(1,N); 
       else
            norm_weight=prev_norm_weight;
       end
    end
    rind=randsample(ind,N,true,norm_weight);
    filtered_states=new_states(:,rind);

    delt_lag=delt;
    prev_norm_weight=norm_weight;
    
    %compute the out-sample predictive densities 
    if(t>Tin)
        pred_r(i,:)=mean(exp(lnormpdf(repmat(ygrid,N,1),repmat(new_J',1,length(ygrid)),repmat(exp(new_x'./2),1,length(ygrid)))));
        pred_bv(i,:)=mean(exp(lnormpdf(repmat(lnBVgrid,N,1),psi0+psi1.*repmat(new_x',1,length(lnBVgrid)),repmat(sigbv,N,length(lnBVgrid)))));
        i=i+1;
    end
end

end

