clear; clc;
addpath('MISC_fn');
addpath('MCMCRoutines');

seed=123456;
rng(seed);
TT=5000; 
%setting for paper
% theta1=0.8; theta2=0.6;
% sig=1;
%robustness #1
% theta1=0.6; theta2=0.2;
% sig=1;
% robustness #2
% theta1=0.6; theta2=0.2;
% sig=sqrt(5);
% robustness #3
theta1=0.8; theta2=0.6;
sig=sqrt(5);
[yy]=simMA2(TT,theta1,theta2,sig);
figure; plot(yy); title('Simulated y');
figure; autocorr(yy);
% save('MA2data.mat');

%% take 500 data points to reproduce Table 2

T=500;
y=yy(1:T);

%number of ABC draws per run
eps=50/(T.^(3/2));
P=500;
reps=round(P/eps);

lt=0; ut=0.99; %draw from prior theta~unif(0,0.99)
ls=0.1; us=3.0;
t1_draw=lt+rand(reps,1)*(ut-lt);
t2_draw=lt+rand(reps,1)*(ut-lt);
s_draw=ls+rand(reps,1)*(us-ls);

%compute the autocovariances of the data up to lag 4
eta_y=autocorr(y,4);
eta_y(1)=var(y); %replace the first element by the variance

%storage vectors for the summary statistics of z
eta_z=zeros(reps,5); %store all for processing later

parfor i=1:reps
    %simulate the pseudo data
   z=simMA2(T,t1_draw(i),t2_draw(i),s_draw(i));
   %compute summary statistics
   acf=autocorr(z,4);
   acf(1)=var(z);
   eta_z(i,:)=acf';
end

%computing the distance based on summary stats
[dist1]= sstatDist_Euclidean(eta_z(:,1:2), eta_y(1:2)', reps);
[dist2]= sstatDist_Euclidean(eta_z(:,1:3), eta_y(1:3)', reps);
[dist3]= sstatDist_Euclidean(eta_z(:,1:4), eta_y(1:4)', reps);
[dist4]= sstatDist_Euclidean(eta_z(:,1:5), eta_y(1:5)', reps);

%sorting the draws based on various statistics
theta_dist1=[t1_draw,t2_draw,s_draw,dist1];
theta_dist1_sorted=sortrows(theta_dist1,4);

theta_dist2=[t1_draw,t2_draw,s_draw,dist2];
theta_dist2_sorted=sortrows(theta_dist2,4);

theta_dist3=[t1_draw,t2_draw,s_draw,dist3];
theta_dist3_sorted=sortrows(theta_dist3,4);

theta_dist4=[t1_draw,t2_draw,s_draw,dist4];
theta_dist4_sorted=sortrows(theta_dist4,4);

%select the chosen draws;
theta_dist1=theta_dist1_sorted(1:P,1:3);
theta_dist2=theta_dist2_sorted(1:P,1:3);
theta_dist3=theta_dist3_sorted(1:P,1:3);
theta_dist4=theta_dist4_sorted(1:P,1:3);

%% Run MCMC for comparison
[~,store_theta,store_sig,~,~,~,~]=MA2_fullMCMC_func(y,theta1,theta2,sig);
tmp=mod(1:20000,40);
take=find(tmp==0);
MCMC_draws=[store_theta(take,:) store_sig(take,:)];

%% Generating predictive scores
% yllim=-10; yulim=10;
yllim=-20; yulim=20; %for robustness#2 and #3
ygrid=linspace(yllim,yulim,100);

%number of out-of-sample obs (to do one-step-ahead predictions)
N=500;
pred1_draws=zeros(P,N);
pred1_kern=zeros(length(ygrid),N);
pred2_draws=zeros(P,N);
pred2_kern=zeros(length(ygrid),N);
pred3_draws=zeros(P,N);
pred3_kern=zeros(length(ygrid),N);
pred4_draws=zeros(P,N);
pred4_kern=zeros(length(ygrid),N);
predMCMC_draws=zeros(P,N);
predMCMC_kern=zeros(length(ygrid),N);

logscores=zeros(N,5);
qscores=zeros(N,5);
crpscores=zeros(N,5);

for i=1:N
    yin=yy(1:T);
    yout=yy(T+1);
    
    % using the ABC draws to obtain predictive density of obs T+1
    % use the same random numbers for all cases for comparability
    eps_t=normrnd(zeros(1,P),ones(1,P));

    %order 1
    [resids] = resids_MA2(yin,theta_dist1(:,1)',theta_dist1(:,2)');
    pred1=theta_dist1(:,3)'.*eps_t+theta_dist1(:,1)'.*resids(T,:)+theta_dist1(:,2)'.*resids(T-1,:);
    pred1_draws(:,i)=pred1;
    pred1_kern(:,i)=Gael_kernel(ygrid,pred1);

    %order 2
    [resids] = resids_MA2(yin,theta_dist2(:,1)',theta_dist2(:,2)');
    pred2=theta_dist2(:,3)'.*eps_t+theta_dist2(:,1)'.*resids(T,:)+theta_dist2(:,2)'.*resids(T-1,:);
    pred2_draws(:,i)=pred2;
    pred2_kern(:,i)=Gael_kernel(ygrid,pred2);

    %order 3
    [resids] = resids_MA2(yin,theta_dist3(:,1)',theta_dist3(:,2)');
    pred3=theta_dist3(:,3)'.*eps_t+theta_dist3(:,1)'.*resids(T,:)+theta_dist3(:,2)'.*resids(T-1,:);
    pred3_draws(:,i)=pred3;
    pred3_kern(:,i)=Gael_kernel(ygrid,pred3);

    %order 4
    [resids] = resids_MA2(yin,theta_dist4(:,1)',theta_dist4(:,2)');
    pred4=theta_dist4(:,3)'.*eps_t+theta_dist4(:,1)'.*resids(T,:)+theta_dist4(:,2)'.*resids(T-1,:);
    pred4_draws(:,i)=pred4;
    pred4_kern(:,i)=Gael_kernel(ygrid,pred4);
    
    [resids] = resids_MA2(yin,MCMC_draws(:,1)',MCMC_draws(:,2)');
    predMCMC=MCMC_draws(:,3)'.*eps_t+MCMC_draws(:,1)'.*resids(T,:)+MCMC_draws(:,2)'.*resids(T-1,:);
    predMCMC_draws(:,i)=predMCMC;
    predMCMC_kern(:,i)=Gael_kernel(ygrid,predMCMC);
    
    %predictive CDF to use in score computation
    pred_kern_cdf1=cumtrapz(ygrid,pred1_kern(:,i));
    pred_kern_cdf1=pred_kern_cdf1/pred_kern_cdf1(end);
    pred_kern_cdf2=cumtrapz(ygrid,pred2_kern(:,i));
    pred_kern_cdf2=pred_kern_cdf2/pred_kern_cdf2(end);
    pred_kern_cdf3=cumtrapz(ygrid,pred3_kern(:,i));
    pred_kern_cdf3=pred_kern_cdf3/pred_kern_cdf3(end);
    pred_kern_cdf4=cumtrapz(ygrid,pred4_kern(:,i));
    pred_kern_cdf4=pred_kern_cdf4/pred_kern_cdf4(end);
    pred_MCMC_cdf=cumtrapz(ygrid,predMCMC_kern(:,i));
    pred_MCMC_cdf=pred_MCMC_cdf/pred_MCMC_cdf(end);

    % compute predictive scores
    [logscores(i,1), qscores(i,1), crpscores(i,1)]=calcPredScoresNum(ygrid,pred1_kern(:,i),pred_kern_cdf1,yout,yllim,yulim);
    [logscores(i,2), qscores(i,2), crpscores(i,2)]=calcPredScoresNum(ygrid,pred2_kern(:,i),pred_kern_cdf2,yout,yllim,yulim);
    [logscores(i,3), qscores(i,3), crpscores(i,3)]=calcPredScoresNum(ygrid,pred3_kern(:,i),pred_kern_cdf3,yout,yllim,yulim);
    [logscores(i,4), qscores(i,4), crpscores(i,4)]=calcPredScoresNum(ygrid,pred4_kern(:,i),pred_kern_cdf4,yout,yllim,yulim);
    [logscores(i,5), qscores(i,5), crpscores(i,5)]=calcPredScoresNum(ygrid,predMCMC_kern(:,i),pred_MCMC_cdf,yout,yllim,yulim);    
    T=T+1;

end

ls_avg=nanmean(logscores);
qs_avg=nanmean(qscores);
crps_avg=nanmean(crpscores);

% save('MA2_robustness3.mat');