function [pred_MCMC,store_theta,store_sig,t1_post_kern,t2_post_kern,s_post_kern,pred_kern_MCMC]=MA2_fullMCMC_func(y,theta1,theta2,sig)
    addpath('E:\Dropbox\Research\ABCForecasting\MISC_fn');
    addpath('MCMCRoutines');
    T=length(y);

    % %start timer 
    % tic;

    %number of MCMC reps
    reps=20000;
    burnin=5000;
    g=reps+burnin;

    %upper and lower bounds for k and sig prior
    lt=0; ut=0.99; %draw from prior theta~unif(0.5,0.99)
    ls=0.1; us=3; %draw from prior sigma~unif(0.1,2)

    %storage vectors
    store_theta=zeros(reps,2);
    store_sig=zeros(reps,1);
    store_lastx=zeros(reps,2);

    %initialize vales
    curr_theta=[theta1 theta2];
    curr_sig=sig;
    curr_x=zeros(T,1);
    prev_x=curr_x;
    theta_accp=0;
    for i=1:g

        %draw the parameters - theta using MH
        [curr_theta,accp]=draw_MA2_theta(y,curr_theta,curr_sig,lt,ut);
        theta_accp=theta_accp+accp;

        curr_sig=draw_MA2_sig(y,curr_theta,ls,us);

        %compute the filtered epsilon
        Hpsi = speye(T) + sparse(2:T,1:(T-1),curr_theta(1)*ones(1,T-1),T,T) ... 
            + sparse(3:T,1:(T-2),curr_theta(2)*ones(1,T-2),T,T);
        curr_x=Hpsi\y;

        if(i>burnin)
           store_theta(i-burnin,:)=curr_theta;
           store_sig(i-burnin)=curr_sig;
           store_lastx(i-burnin,:)=curr_x(end-1:end)';
        end
        prev_x=curr_x; %need this for MH of the state
        if(mod(i,100)==0)
            disp([' MCMC iteration ' num2str(i) ', theta=' num2str(curr_theta) ', sig=' num2str(curr_sig) ]);
        end
    end
    theta_MH=theta_accp/g;

    %% extracting the posterior predictive distribution for y_T+1
    %using thinned out draws - take every 20th draw 
    tmp=mod(1:reps,40);
    take=find(tmp==0);
    ygrid=linspace(-10,10,100);

    %simulation-based
    [resids_MCMC] = resids_MA2(y,store_theta(take,1)',store_theta(take,2)');
    eps_t=normrnd(zeros(1,length(take)),store_sig(take)');
    pred_MCMC=eps_t+store_theta(take,1)'.*resids_MCMC(T,:)+store_theta(take,2)'.*resids_MCMC(T-1,:);
    pred_kern_MCMC=Gael_kernel(ygrid,pred_MCMC);

    thetag=linspace(lt,ut,100);
    [t1_post_kern]=Gael_kernel(thetag,store_theta(take,1));
    [t2_post_kern]=Gael_kernel(thetag,store_theta(take,2));
    sigg=linspace(ls,us,100);
    [s_post_kern]=Gael_kernel(sigg,store_sig(take));
end