function [ x ] = cigrnd( alpha,beta,llim,ulim )
%Generates from an IG(alpha,beta) constrained from above, below or both.
% The method used is to recognise that if X ~ IG(a,b), 
% then Y=2.b/X ~ Chi^2(df=2.a), so first generate Y from a constrained
% Chi^2(2.a) and transform back to IG.
% (Note limits are inverted because Y=2.b/X).

if(ulim<llim)
 error('Fatal Error are cigrnd: upper limits <= lower limits');
end;

if(isinf(ulim))
    fa=0; %lower limit for chisq set to zero
else
    fa=gamcdf(1/ulim,alpha,1/beta); %lower limit of gamma set to F(a),
                                    % a=1/ulim, F(.) cdf of G(alpha,1/beta)
end

if((llim<=0)||isinf(llim))
    fb=1; %upper limit for chisq set to F(b), b=2.beta/llim
else
    fb=gamcdf(1/llim,alpha,1/beta); %upper limit of gamma set to F(b),
                                    %b=1/llim, F(.) cdf of G(alpha,1/beta)
end

uni=rand*(fb-fa)+fa;
x=1/gaminv(uni,alpha,1/beta);

end

